
local tremove	= table.remove
local Map			= {}
Map.__index		= Map
setmetatable(Map, {__call = function(cls, ...) return cls.new(...) end})

local TileLayer		= require("Editor.TileLayer")
local ChunkLoader	= require("Editor.ChunkLoader")
local Tileset 		= require("Editor.Tileset")



------------------------------------------------------------|
-- Local Functions --
------------------------------------------------------------|
local function setupLayers(self, data)
	if not data then
		data = {}
		data[1] = {type = "TileLayer", name = "Background", id = 1, chunksettings = self.chunksettings}
		data[2] = {type = "TileLayer", name = "TileLayer1", id = 2, chunksettings = self.chunksettings}
	end

	local layers = {}
	for i=1, #data do
		if data[i].type == "TileLayer" then
			layers[i] = TileLayer(data[i], self)
		end
	end
	return layers
end



------------------------------------------------------------|
-- Map Class --
------------------------------------------------------------|
function Map.new(data)
	local self		= setmetatable({}, Map)
	local data		= data or {}
	self.tileset	= Tileset(data.tileset or {imagepath="tileset.png", tilesize=32})
	self.tilesize	= self.tileset:getTilesize()
	self.chunksettings = data.chunksettings or {cw=24, ch=24, tw=32, th=32}
	self.width		= self.chunksettings.cw*self.chunksettings.tw
	self.height		= self.chunksettings.ch*self.chunksettings.th
	self.layers		= setupLayers(self, data.layers)
	self:setLayer(#self.layers)
	return self
end



function Map:getSaveData()
	local data = {}
	-- Tileset --
	data.tileset	= self.tileset:getSaveData()
	-- Layers --
	data.layers	= {}
	for i=1, #self.layers do data.layers[i] = self.layers[i]:getSaveData() end
	-- Map --
	data.chunksettings = self.chunksettings
	return data
end



-- function Map:addCustomLayer(type, name, data)
-- 	local data		= data or {}
-- 	local layer		= Layer(name, self.width, self.height, self:getTilesize(), data)
-- 	self.layers[#self.layers+1] = layer
-- end



function Map:toTileCoords(x, y)
  return math.floor(x/self.tilesize)+1, math.floor(y/self.tilesize)+1
end



function Map:toWorldCoords(x, y)
  return x*self.tilesize-self.tilesize, y*self.tilesize-self.tilesize
end



function Map:setTile(layer, x, y, tid)
	if not layer.tiles[y] then return
	elseif not layer.tiles[y][x] then return end
	if layer.tiles[y][x] ~= tid then
		layer.tiles[y][x] = tid
		print("updating tile "..x..", "..y)
		layer.chunks:updateChunks({{x, y}})
	end
end



function Map:setTiles(layer, tiles)
	for i=#tiles, 1, -1 do
		local tx, ty, tid = tiles[i][1], tiles[i][2], tiles[i][3]
		if layer.tiles[ty] and layer.tiles[ty][tx] and layer.tiles[ty][tx] ~= tid then
				layer.tiles[ty][tx] = tid
				print("updating tile "..tx..", "..ty)
		else tremove(tiles, i) end
	end
	if #tiles > 0 then layer.chunks:updateChunks(tiles) end
end



function Map:getTile(layer, x, y)
	if not layer.tiles[y] then return
	elseif not layer.tiles[y][x] then return end
	return layer.tiles[y][x]
end



function Map:setLayer(id)
	self.selectedLayer = self:getLayer(id)
	print("Selected Layer: "..tostring(self.selectedLayer.name))
end



function Map:getLayer(id)
	if type(id) == "string" then
		for i=1, #self.layers do
			local layer = self.layers[i]
			if layer.name == id then return layer end
		end
	end
	return self.layers[id]
end



function Map:update(dt)
	--> update animated tiles
end



function Map:draw(camera)
	-- Draw Layers --
	for i=1, #self.layers do
		self.layers[i]:draw(camera)
	end
	--> draw animated tiles
end

return Map
