
local TileLayer		= {}
TileLayer.__index	= TileLayer
setmetatable(TileLayer, {__call = function(cls, ...) return cls.new(...) end})

local ChunkLoader	= require("Editor.ChunkLoader")



------------------------------------------------------------|
-- TileLayer Class --
------------------------------------------------------------|
function TileLayer.new(data, map)
	local self = setmetatable({}, TileLayer)
	local data			= data or {}
	self.id					= data.id
	self.type				= "TileLayer"
	self.name 			= data.name or "Default"
	self.color			= data.color or {255, 255, 255}
	self.visible 		= data.visible or true
	self.opacity 		= data.opacity or 255
	self.map				= map
	self.tiles 			= data.tiles or {}

	-- Pre-generate tiles
	if not self.tiles[1] then
		for y = 1, map.height do
			self.tiles[y] = {}
			for x = 1, map.width do
				if self.name == "Background" then self.tiles[y][x] = math.random(17, 20)
				else self.tiles[y][x] = 0 end
			end
		end
	end

	self.chunks	= ChunkLoader(self, map.chunksettings)
	return self
end



function TileLayer:getSaveData()
	return {
		id			= self.id,
		type 		= self.type,
		name 		= self.name,
		color 	= self.color,
		visible = self.visible,
		opacity = self.opacity,
		tiles		= self.tiles,
	}
end



function TileLayer:draw(camera)
	 return self.chunks:draw(camera)
end

return TileLayer
