

local Tileset = {}
Tileset.__index = Tileset
setmetatable(Tileset, {__call = function(cls, ...) return cls.new(...) end})


------------------------------------------------------------|
-- Local Functions --
------------------------------------------------------------|
local function createTiles(imagepath, tilesize, properties)
	if not imagepath then return {[0] = {}} end
	local image				= love.graphics.newImage(imagepath)
	local iw, ih			= image:getDimensions()
	local properties	= properties or {}
	local amount			= iw/tilesize * ih/tilesize
	local tiles				= {[0] = {}}
	local rowsize 		= iw/tilesize

	for i = 1, amount do
		local row				= math.floor((i-1)/rowsize)
		local column		= (i-1)%rowsize
		local tile			= {}
		local x					= column*tilesize
		local y					= row*tilesize
		tile.quad				= love.graphics.newQuad(x, y, tilesize, tilesize, iw, ih)
		tile.properties	= properties[i] or {color={255, 255, 255, 255}}
		table.insert(tiles, tile)
	end

	return tiles, image
end



------------------------------------------------------------|
-- Tileset Class --
------------------------------------------------------------|
function Tileset.new(tileset)
	local self = setmetatable({}, Tileset)
	local tileset		= tileset or {}
	self.imagepath	= tileset.imagepath
	self.properties	= tileset.properties
	self.tilesize 	= tileset.tilesize or 32

	if self.imagepath then
		self.tiles, self.image = createTiles(self.imagepath, self.tilesize, self.properties)
	else self.tiles = createTiles() end

	return self
end



function Tileset:getSaveData()
	return {
		imagepath		= self.imagepath,
		properties 	= self.properties,
		tilesize 		= self.tilesize,
	}
end



function Tileset:getTile(tid)
	return self.tiles[tid]
end



function Tileset:getTiles()
	return self.tiles
end



function Tileset:getImage()
	return self.image
end



function Tileset:getTilesize()
	return self.tilesize
end

return Tileset
