

local UI = {}
UI.__index = UI
setmetatable(UI, {__call = function(cls, ...) return cls.new(...) end})

local Tileselector 		= require("Editor.ui.Tileselector")



------------------------------------------------------------|
-- Local Functions --
------------------------------------------------------------|
local function aabb(a, b)
	return a.x < b.x+b.width and a.x + a.width > b.x and
	a.y < b.y+b.height and a.y + a.height > b.y
end



------------------------------------------------------------|
-- UI --
------------------------------------------------------------|
function UI.new(map)
	local self = setmetatable({}, UI)
	self.elements			= {}
	self.ocupied			= false
	self.tileselector = self:addElement(Tileselector(0, 200, 5, map.tileset))
	return self
end



function UI:isOcupied()
	local mouse		= {x=love.mouse.getX(), y=love.mouse.getY(), width=1, height=1}
	for i=1, #self.elements do
		local element = self.elements[i]
		if aabb(element, mouse) then return true end
	end
	return false
end



function UI:setSelectedTileID(tid)
	self.tileselector:setSelectedTileID(tid)
end



function UI:getSelectedTileID()
	return self.tileselector:getSelectedTileID()
end



function UI:addElement(element)
	self.elements[#self.elements+1] = element
	return element
end



function UI:update(dt)
	for i=1, #self.elements do
		local element = self.elements[i]
		if element.update then element:update(dt) end
	end
end



function UI:draw()
	for i=1, #self.elements do
		local element = self.elements[i]
		if element.draw then element:draw() end
	end
end



function UI:mousepressed(x, y, button)
	for i=1, #self.elements do
		local element = self.elements[i]
		if element.mousepressed then element:mousepressed(x, y, button) end
	end
end

return UI
