

local Tileselector		= {}
Tileselector.__index	= Tileselector
setmetatable(Tileselector, {__call = function(cls, ...) return cls.new(...) end})



------------------------------------------------------------|
-- Local Functions --
------------------------------------------------------------|
local function aabb(a, b)
	return a.x < b.x+b.width and a.x + a.width > b.x and
	a.y < b.y+b.height and a.y + a.height > b.y
end



local function createTileButtons(tileselector)
	local buttons = {}
	for i=1, #tileselector.tiles do
		local button	= {}
		local row			= math.floor((i-1)/tileselector.tilesInRow)
		local column	= (i-1)%tileselector.tilesInRow
		local x				= column*tileselector.tw
		local y				= row*tileselector.tw
		button.x			= tileselector.x+x
		button.y			= tileselector.y+y
		button.tid		= i
		button.width	= tileselector.tw
		button.height	= tileselector.tw
		button.tile		= tileselector.tiles[i]
		buttons[i]		= button
	end
	return buttons
end



------------------------------------------------------------|
-- Tileselector --
------------------------------------------------------------|
function Tileselector.new(x, y, tilesInRow, tileset)
	local self 				= setmetatable({}, Tileselector)

	self.x						= x
	self.y						= y
	self.tiles				= tileset:getTiles()
	self.tw						= tileset:getTilesize()
	self.tilesInRow		= tilesInRow
	self.width				=	self.tilesInRow*self.tw
	self.height				=	(#self.tiles/self.tilesInRow)*self.tw

	self.selectedTileID	= 21

	self.tilesetImage	= tileset:getImage()
	self.tileButtons	= createTileButtons(self)
	return self
end



function Tileselector:setSelectedTileID(tid)
	self.selectedTileID = tid
end



function Tileselector:getSelectedTileID()
	return self.selectedTileID
end



function Tileselector:update(dt)
	for i=1, #self.tileButtons do
		local button	= self.tileButtons[i]
		local mx, my	= love.mouse.getPosition()
		local mouse		= {x=mx, y=my, width=1, height=1}
		if aabb(button, mouse) then
			button.selected = true
		else
			button.selected = false
		end
	end
end



function Tileselector:draw()
	local previousColor = {love.graphics.getColor()}
	love.graphics.setColor(255, 255, 255, 10)
	love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)
	for i=1, #self.tileButtons do
		local button				= self.tileButtons[i]
		if button.selected then love.graphics.setColor(255, 255, 255, 255)
		else love.graphics.setColor(175, 175, 175, 175) end
		love.graphics.draw(self.tilesetImage, button.tile.quad, button.x, button.y)
	end
	love.graphics.setColor(previousColor)
end



function Tileselector:mousepressed(x, y, button)
	for i=1, #self.tileButtons do
		local tileButton = self.tileButtons[i]
		if tileButton.selected and button == 1 then
			self:setSelectedTileID(tileButton.tid)
		end
	end
end

return Tileselector
